@@ -21,6 +21,7 @@
 #' @aliases QIC QIC.geeglm QIC.geekin QIC.ordgee
 #' @param object a fitted GEE model from the geepack package. Currently only
 #' works on geeglm objects
+#' @param tol the tolerance used for matrix inversion
 #' @param \dots optionally more fitted geeglm model objects
 #' @return A vector or matrix with the QIC, QICu, quasi likelihood, CIC, the
 #' number of mean effect parameters, and the corrected QIC for each GEE object
@@ -45,7 +46,7 @@
 #'
 #' @rdname QIC
 #' @export
-QIC.geeglm <- function(object, ...) {
+QIC.geeglm <- function(object, tol=.Machine$double.eps, ...) {
 
   #
   # The majority of this code was taken from the internet
@@ -82,11 +83,11 @@ QIC.geeglm <- function(object, ...) {
     # Fit model with independence correlation structure
     object$call$corstr <- "independence"
     object$call$zcor <- NULL
-    model.indep <- eval(object, parent.frame())
+    model.indep <- eval(object$call, parent.frame())
     # model.indep <- update(object, corstr="independence",zcorr=NULL)
 
     # Trace term (penalty for model complexity)
-    AIinverse <- invert(model.indep$geese$vbeta.naiv)
+    AIinverse <- invert(model.indep$geese$vbeta.naiv, tol=tol)
     Vr <- object$geese$vbeta
     trace <- sum(diag(AIinverse %*% Vr))
     params <- length(coef(object)) # Mean parameters in the model
@@ -131,7 +132,7 @@ QIC.geeglm <- function(object, ...) {
 
 #' @rdname QIC
 #' @export
-QIC.ordgee <- function(object, ...) {
+QIC.ordgee <- function(object, tol = .Machine$double.eps, ...) {
 
   #
   # The majority of this code was taken from the internet
@@ -318,7 +319,7 @@ QIC.ordgee <- function(object, ...) {
 
 #' @rdname QIC
 #' @export
-QIC.geekin <- function(object, ...) {
+QIC.geekin <- function(object,  tol = .Machine$double.eps, ...) {
 
   # This functions is only needed to replace class
   # geeglm to make sure the regular
@@ -340,6 +341,6 @@ QIC.geekin <- function(object, ...) {
 
 #' @rdname QIC
 #' @export
-QIC <- function(object, ...) {
+QIC <- function(object,  tol = .Machine$double.eps, ...) {
   UseMethod("QIC")
 }
